==> skrooge_0.6.0.orig.tar.gz: skrooge-0.6.0/skrooge_scheduled/skgscheduledpluginwidget.cpp <==
/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * A skrooge plugin to manage scheduled operations.
 *
 * @author Stephane MANKOWSKI
 */
#include "skgscheduledpluginwidget.h"
#include "skgmainpanel.h"
#include "skgtraces.h"
#include "skgdocumentbank.h"
#include "skgobjectmodel.h"
#include "skgsortfilterproxymodel.h"
#include "skgrecurrentoperationobject.h"
#include "skgtransactionmng.h"

#include <klineedit.h>

#include <QDomDocument>
#include <QHeaderView>
#include <QKeyEvent>

SKGScheduledPluginWidget::SKGScheduledPluginWidget ( SKGDocumentBank* iDocument )
        :SKGTabPage ( iDocument )
{
    SKGTRACEIN ( 1, "SKGScheduledPluginWidget::SKGScheduledPluginWidget" );

    ui.setupUi ( this );

    //Build you panel here
    SKGObjectModel* modelview = new SKGObjectModel ( ( SKGDocumentBank* ) getDocument(), "v_recurrentoperation_display", "", this, "", false );
    SKGSortFilterProxyModel* modelproxy = new SKGSortFilterProxyModel ( this );
    modelproxy->setSourceModel ( modelview );
    modelproxy->setSortRole ( Qt::UserRole );
    modelproxy->setDynamicSortFilter ( true );

    ui.kFilterEdit->setProxy ( modelproxy );

    ui.kView->setModel ( modelproxy );

    connect ( ui.kView, SIGNAL ( selectionChangedDelayed() ), this, SLOT ( onSelectionChanged() ) );

    connect ( modelview, SIGNAL ( beforeReset() ), ui.kView, SLOT ( saveSelection() ) );
    connect ( modelview, SIGNAL ( afterReset() ), ui.kView, SLOT ( resetSelection() ) );

    //Add registered global action in contextual menu

    if ( SKGMainPanel::getMainPanel() ) ui.kView->insertAction ( 0, SKGMainPanel::getMainPanel()->getGlobalAction ( "edit_delete" ) );

    //Add Standard KDE Icons to buttons to Operations
    ui.kModifyBtn->setIcon ( KIcon ( "dialog-ok-apply" ) );
    ui.kProcessBtn->setIcon ( KIcon ( "system-run" ) );

    ui.kTitle->setPixmap ( KIcon ( "dialog-information" ).pixmap ( 22, 22 ), KTitleWidget::ImageLeft );

    //Set Event filters to catch CTRL+ENTER or SHIFT+ENTER
    this->installEventFilter ( this );

    //Refresh
    connect ( ( const QObject* ) getDocument(), SIGNAL ( tableModified ( QString, int ) ), this, SLOT ( dataModified ( QString, int ) ), Qt::QueuedConnection );
    dataModified ( "", 0 );
}

SKGScheduledPluginWidget::~SKGScheduledPluginWidget()
{
    SKGTRACEIN ( 1, "SKGScheduledPluginWidget::~SKGScheduledPluginWidget" );
}

bool SKGScheduledPluginWidget::eventFilter ( QObject *object, QEvent *event )
{
    if ( event && event->type() == QEvent::KeyPress  )
    {
      QKeyEvent *keyEvent = ( QKeyEvent * ) event;
      if ( (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) && object==this)
      {
	if ( QApplication::keyboardModifiers() &Qt::ShiftModifier && ui.kModifyBtn->isEnabled())
	{
	    ui.kModifyBtn->click();
	}
      }
    }

    return false;
}

SKGObjectBase::SKGListSKGObjectBase SKGScheduledPluginWidget::getSelectedObjects()
{
    return ui.kView->getSelectedObjects();
}

int SKGScheduledPluginWidget::getNbSelectedObjects()
{
    return ui.kView->getNbSelectedObjects();
}

QString SKGScheduledPluginWidget::getState()
{
    SKGTRACEIN ( 10, "SKGScheduledPluginWidget::getState" );
    QDomDocument doc ( "SKGML" );
    QDomElement root = doc.createElement ( "parameters" );
    doc.appendChild ( root );

    root.setAttribute ( "filter", ui.kFilterEdit->lineEdit()->text() );

    //Memorize table settings
    root.setAttribute ( "view", ui.kView->getState() );

    return doc.toString();
}

void SKGScheduledPluginWidget::setState ( const QString& iState )
{
    SKGTRACEIN ( 10, "SKGScheduledPluginWidget::setState" );
    QDomDocument doc ( "SKGML" );
    doc.setContent ( iState );
    QDomElement root = doc.documentElement();

    QString filter=root.attribute ( "filter" );
    if ( !filter.isEmpty() ) ui.kFilterEdit->setText ( filter );

    ui.kView->setState ( root.attribute ( "view" ) );

    //Correction bug 2299394 vvv
    if ( ui.kView->isAutoResized() ) ui.kView->resizeColumnsToContents();
    //Correction bug 2299394 ^^^
}

QString SKGScheduledPluginWidget::getDefaultStateAttribute()
{
    return "SKGSCHEDULED_DEFAULT_PARAMETERS";
}

QWidget* SKGScheduledPluginWidget::getWidgetForPrint()
{
    return ui.kView;
}

void SKGScheduledPluginWidget::dataModified ( const QString& iTableName, int iIdTransaction )
{
    SKGTRACEIN ( 1, "SKGScheduledPluginWidget::dataModified" );
    Q_UNUSED ( iIdTransaction );

    if ( iTableName=="recurrentoperation" || iTableName.isEmpty() )
    {
        //Correction bug 2299394 vvv
        if ( ui.kView->isAutoResized() ) ui.kView->resizeColumnsToContents();
        //Correction bug 2299394 ^^^

        //Correction 215658: vvv because the table is modified, the selection is modified
        onSelectionChanged();
        //Correction 215658: ^^^
    }
}

void SKGScheduledPluginWidget::onDoubleClick()
{
    _SKGTRACEIN ( 10, "SKGScheduledPluginWidget::onDoubleClick" );

    //Get selection
    SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();
    if ( selection.count() ==1 )
    {
        SKGRecurrentOperationObject recOp ( selection.at ( 0 ) );

        //Build where clause and title
        QString wc="((SELECT count(1) FROM v_recurrentoperation s WHERE s.rd_operation_id=v_operation_display.id and s.id="+SKGServices::intToString ( recOp.getID() ) +")>0) OR r_recurrentoperation_id="+SKGServices::intToString ( recOp.getID() );
        QString title=i18nc("Noun, a list of items", "Recurrent operations" );

        //Open
        if ( QApplication::keyboardModifiers() &Qt::ControlModifier && QApplication::keyboardModifiers() &Qt::ShiftModifier )
        {
            //Call debug plugin
            QDomDocument doc ( "SKGML" );
            QDomElement root = doc.createElement ( "parameters" );
            doc.appendChild ( root );
            root.setAttribute ( "sqlOrder", "SELECT * from v_operation_display WHERE "+wc );

            SKGMainPanel::getMainPanel()->setNewTabContent ( SKGMainPanel::getMainPanel()->getPluginByName ( "Skrooge debug plugin" ), -1, doc.toString() );
        }
        else
        {
            //Call operation plugin
            QDomDocument doc ( "SKGML" );
            doc.setContent ( getDocument()->getParameter ( "SKGOPERATION_DEFAULT_PARAMETERS" ) );
            QDomElement root = doc.documentElement();
            if ( root.isNull() )
            {
                root=doc.createElement ( "parameters" );
                doc.appendChild ( root );
            }

            root.setAttribute ( "operationTable", "v_operation_display" );
            root.setAttribute ( "operationWhereClause", wc );
            root.setAttribute ( "title", title );
            root.setAttribute ( "title_icon", "chronometer" );
            root.setAttribute ( "currentPage", "-1" );

            SKGMainPanel::getMainPanel()->setNewTabContent ( SKGMainPanel::getMainPanel()->getPluginByName ( "Skrooge operation plugin" ), -1, doc.toString() );
        }
    }
}

void SKGScheduledPluginWidget::onSelectionChanged()
{
    SKGTRACEIN ( 10, "SKGScheduledPluginWidget::onSelectionChanged" );

    int nb=getNbSelectedObjects();
    ui.kFirstOccurenceDate->setEnabled ( nb<=1 );
    ui.kModifyBtn->setEnabled ( nb );
    ui.kProcessBtn->setEnabled ( nb );

    if ( nb==1 )
    {
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();
        SKGRecurrentOperationObject recOp=selection.at ( 0 );

        ui.kFirstOccurenceDate->setDate ( recOp.getDate() );
        ui.kOnceEveryVal->setValue ( recOp.getPeriodIncrement() );
        ui.kOnceEveryUnit->setCurrentIndex ( ( int ) recOp.getPeriodUnit() );

        ui.kRemindMeVal->setValue ( recOp.getWarnDays() );
        ui.kRemindMe->setCheckState ( recOp.isWarnEnabled() ? Qt::Checked : Qt::Unchecked );

        ui.kAutoWriteVal->setValue ( recOp.getAutoWriteDays() );
        ui.kAutoWrite->setCheckState ( recOp.isAutoWriteEnabled() ? Qt::Checked : Qt::Unchecked );

        ui.kNbTimesVal->setValue ( recOp.getTimeLimit() );
        ui.kNbTimes->setCheckState ( recOp.hasTimeLimit() ? Qt::Checked : Qt::Unchecked );
    }

    emit selectionChanged();
}

void SKGScheduledPluginWidget::onNbOccurrenceChanged()
{
    QDate firstDate=ui.kFirstOccurenceDate->date();
    SKGRecurrentOperationObject::PeriodUnit punit= ( SKGRecurrentOperationObject::PeriodUnit ) ui.kOnceEveryUnit->currentIndex();
    int p=ui.kOnceEveryVal->value();

    if ( ui.kLastOccurenceDate==this->sender() )
    {
        //End date has been modified.
        //We must set the number of occurrence
        QDate lastDate=ui.kLastOccurenceDate->date();
        if ( lastDate<=firstDate )
        {
            ui.kLastOccurenceDate->setDate ( firstDate );
            ui.kNbTimesVal->setValue ( 1 );
        }
        else
        {
            int nbd=firstDate.daysTo ( lastDate );
            if ( punit==SKGRecurrentOperationObject::DAY ) nbd=nbd/p;
            else if ( punit==SKGRecurrentOperationObject::MONTH )
            {
                nbd= ( lastDate.day() >=firstDate.day() ? 0 : -1 ) + ( lastDate.year()-firstDate.year() ) *12+ ( lastDate.month()-firstDate.month() );
            }
            else if ( punit==SKGRecurrentOperationObject::YEAR ) nbd=nbd/ ( 365*p );

            bool previous=ui.kNbTimesVal->blockSignals ( true );
            ui.kNbTimesVal->setValue ( nbd+1 );
            ui.kNbTimesVal->blockSignals ( previous );
        }

    }
    else
    {
        //We must compute the date
        p*= ( ui.kNbTimesVal->value()-1 );
        if ( punit==SKGRecurrentOperationObject::DAY ) firstDate=firstDate.addDays ( p );
        else if ( punit==SKGRecurrentOperationObject::MONTH ) firstDate=firstDate.addMonths ( p );
        else if ( punit==SKGRecurrentOperationObject::YEAR ) firstDate=firstDate.addYears ( p );

        bool previous=ui.kLastOccurenceDate->blockSignals ( true );
        ui.kLastOccurenceDate->setDate ( firstDate );
        ui.kLastOccurenceDate->blockSignals ( previous );
    }
}

void SKGScheduledPluginWidget::onUpdate()
{
    SKGError err;
    SKGTRACEINRC ( 10, "SKGScheduledPluginWidget::onUpdate",err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        //Get Selection
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

        int nb=selection.count();
        SKGBEGINPROGRESSTRANSACTION(*getDocument(), i18nc("Noun, name of the user action", "Recurrent operation update" ), err, nb );
        for ( int i=0; err.isSucceeded() && i<nb; ++i )
        {
            //Get the real object, not the object from the view
            SKGRecurrentOperationObject recOp=SKGRecurrentOperationObject ( selection.at ( i ).getDocument(), selection.at ( i ).getID() );

            //Update it
            if ( nb==1 ) err=recOp.setDate ( ui.kFirstOccurenceDate->date() );
            if ( err.isSucceeded() ) err=recOp.setPeriodIncrement ( ui.kOnceEveryVal->value() );
            if ( err.isSucceeded() ) err=recOp.setPeriodUnit ( ( SKGRecurrentOperationObject::PeriodUnit ) ui.kOnceEveryUnit->currentIndex() );
            if ( err.isSucceeded() ) err=recOp.setWarnDays ( ui.kRemindMeVal->value() );
            if ( err.isSucceeded() ) err=recOp.warnEnabled ( ui.kRemindMe->checkState() ==Qt::Checked );
            if ( err.isSucceeded() ) err=recOp.setAutoWriteDays ( ui.kAutoWriteVal->value() );
            if ( err.isSucceeded() ) err=recOp.autoWriteEnabled ( ui.kAutoWrite->checkState() ==Qt::Checked );
            if ( err.isSucceeded() ) err=recOp.setTimeLimit ( ui.kNbTimesVal->value() );
            if ( err.isSucceeded() ) err=recOp.timeLimit ( ui.kNbTimes->checkState() ==Qt::Checked );
            if ( err.isSucceeded() ) err=recOp.save();

            if ( err.isSucceeded() ) err=getDocument()->stepForward ( i+1 );
        }
        //status bar
        if ( err.isSucceeded() ) err=SKGError(0, i18nc("Successful message after an user action", "Recurrent operation updated." ) );
        else err.addError ( ERR_FAIL, i18nc("Error message",  "Update failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGScheduledPluginWidget::onProcess()
{
    SKGError err;
    SKGTRACEINRC ( 10, "SKGScheduledPluginWidget::onProcess",err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        //Get Selection
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

        int nb=selection.count();
        SKGBEGINPROGRESSTRANSACTION(*getDocument(), i18nc("Noun, name of the user action", "Insert recurrent operations" ), err, nb );
        for ( int i=0; err.isSucceeded() && i<nb; ++i )
        {
            //Get the real object, not the object from the view
            SKGRecurrentOperationObject recOp=SKGRecurrentOperationObject ( selection.at ( i ).getDocument(), selection.at ( i ).getID() );

            //Process it
            int nbi=0;
            err=recOp.process ( nbi, true );
            if ( err.isSucceeded() ) err=getDocument()->stepForward ( i+1 );
        }
        //status bar
        if ( err.isSucceeded() ) err=SKGError(0, i18nc("Successful message after an user action", "Recurrent operation inserted." ) );
        else err.addError ( ERR_FAIL, i18nc("Error message",  "Insertion failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

#include "skgscheduledpluginwidget.moc"

==> skrooge_0.7.2.2.orig.tar.gz: skrooge-0.7.2.2/skrooge_scheduled/skgscheduledpluginwidget.cpp <==
/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * A skrooge plugin to manage scheduled operations.
 *
 * @author Stephane MANKOWSKI
 */
#include "skgscheduledpluginwidget.h"
#include "skgmainpanel.h"
#include "skgtraces.h"
#include "skgdocumentbank.h"
#include "skgobjectmodel.h"
#include "skgsortfilterproxymodel.h"
#include "skgrecurrentoperationobject.h"
#include "skgtransactionmng.h"

#include <klineedit.h>

#include <QDomDocument>
#include <QHeaderView>
#include <QKeyEvent>

SKGScheduledPluginWidget::SKGScheduledPluginWidget ( SKGDocumentBank* iDocument )
        :SKGTabPage ( iDocument )
{
    SKGTRACEIN ( 1, "SKGScheduledPluginWidget::SKGScheduledPluginWidget" );

    ui.setupUi ( this );

    //Build you panel here
    SKGObjectModel* modelview = new SKGObjectModel ( static_cast<SKGDocumentBank*>(getDocument()), "v_recurrentoperation_display", "", this, "", false );
    SKGSortFilterProxyModel* modelproxy = new SKGSortFilterProxyModel ( this );
    modelproxy->setSourceModel ( modelview );
    modelproxy->setSortRole ( Qt::UserRole );
    modelproxy->setDynamicSortFilter ( true );

    ui.kFilterEdit->setProxy ( modelproxy );

    ui.kView->setModel ( modelproxy );

    connect ( ui.kView, SIGNAL ( selectionChangedDelayed() ), this, SLOT ( onSelectionChanged() ) );

    connect ( modelview, SIGNAL ( beforeReset() ), ui.kView, SLOT ( saveSelection() ) );
    connect ( modelview, SIGNAL ( afterReset() ), ui.kView, SLOT ( resetSelection() ) );

    //Add registered global action in contextual menu

    if ( SKGMainPanel::getMainPanel() ) ui.kView->insertGlobalAction ( "edit_delete" );

    //Add Standard KDE Icons to buttons to Operations
    ui.kModifyBtn->setIcon ( KIcon ( "dialog-ok-apply" ) );
    ui.kProcessBtn->setIcon ( KIcon ( "system-run" ) );

    ui.kTitle->setPixmap ( KIcon ( "dialog-information" ).pixmap ( 22, 22 ), KTitleWidget::ImageLeft );

    //Set Event filters to catch CTRL+ENTER or SHIFT+ENTER
    this->installEventFilter ( this );

    //Refresh
    connect ( ( const QObject* ) getDocument(), SIGNAL ( tableModified ( QString, int ) ), this, SLOT ( dataModified ( QString, int ) ), Qt::QueuedConnection );
    dataModified ( "", 0 );
}

SKGScheduledPluginWidget::~SKGScheduledPluginWidget()
{
    SKGTRACEIN ( 1, "SKGScheduledPluginWidget::~SKGScheduledPluginWidget" );
}

bool SKGScheduledPluginWidget::eventFilter ( QObject *object, QEvent *event )
{
    if ( event && event->type() == QEvent::KeyPress  )
    {
        QKeyEvent *keyEvent = ( QKeyEvent * ) event;
        if ( (keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter) && object==this)
        {
            if ( QApplication::keyboardModifiers() &Qt::ShiftModifier && ui.kModifyBtn->isEnabled())
            {
                ui.kModifyBtn->click();
            }
        }
    }

    return false;
}

SKGObjectBase::SKGListSKGObjectBase SKGScheduledPluginWidget::getSelectedObjects()
{
    return ui.kView->getSelectedObjects();
}

int SKGScheduledPluginWidget::getNbSelectedObjects()
{
    return ui.kView->getNbSelectedObjects();
}

QString SKGScheduledPluginWidget::getState()
{
    SKGTRACEIN ( 10, "SKGScheduledPluginWidget::getState" );
    QDomDocument doc ( "SKGML" );
    QDomElement root = doc.createElement ( "parameters" );
    doc.appendChild ( root );

    root.setAttribute ( "filter", ui.kFilterEdit->lineEdit()->text() );

    //Memorize table settings
    root.setAttribute ( "view", ui.kView->getState() );

    return doc.toString();
}

void SKGScheduledPluginWidget::setState ( const QString& iState )
{
    SKGTRACEIN ( 10, "SKGScheduledPluginWidget::setState" );
    QDomDocument doc ( "SKGML" );
    doc.setContent ( iState );
    QDomElement root = doc.documentElement();

    QString filter=root.attribute ( "filter" );
    if ( !filter.isEmpty() ) ui.kFilterEdit->setText ( filter );

    ui.kView->setState ( root.attribute ( "view" ) );

    //Correction bug 2299394 vvv
    if ( ui.kView->isAutoResized() ) ui.kView->resizeColumnsToContentsDelayed();
    //Correction bug 2299394 ^^^
}

QString SKGScheduledPluginWidget::getDefaultStateAttribute()
{
    return "SKGSCHEDULED_DEFAULT_PARAMETERS";
}

QWidget* SKGScheduledPluginWidget::getWidgetForPrint()
{
    return ui.kView;
}

void SKGScheduledPluginWidget::dataModified ( const QString& iTableName, int iIdTransaction )
{
    SKGTRACEIN ( 1, "SKGScheduledPluginWidget::dataModified" );
    Q_UNUSED ( iIdTransaction );

    if ( iTableName=="recurrentoperation" || iTableName.isEmpty() )
    {
        //Correction bug 2299394 vvv
        if ( ui.kView->isAutoResized() ) ui.kView->resizeColumnsToContentsDelayed();
        //Correction bug 2299394 ^^^

        //Correction 215658: vvv because the table is modified, the selection is modified
        onSelectionChanged();
        //Correction 215658: ^^^
    }
}

void SKGScheduledPluginWidget::onDoubleClick()
{
    _SKGTRACEIN ( 10, "SKGScheduledPluginWidget::onDoubleClick" );

    //Get selection
    SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();
    if ( selection.count() ==1 )
    {
        SKGRecurrentOperationObject recOp ( selection.at ( 0 ) );

        //Build where clause and title
        QString wc="EXISTS(SELECT 1 FROM recurrentoperation s WHERE s.rd_operation_id=v_operation_display.id and s.id="+SKGServices::intToString ( recOp.getID() ) +") OR r_recurrentoperation_id="+SKGServices::intToString ( recOp.getID() );
        QString title=i18nc("Noun, a list of items", "Recurrent operations" );

        //Open
        if ( QApplication::keyboardModifiers() &Qt::ControlModifier && QApplication::keyboardModifiers() &Qt::ShiftModifier )
        {
            //Call debug plugin
            QDomDocument doc ( "SKGML" );
            QDomElement root = doc.createElement ( "parameters" );
            doc.appendChild ( root );
            root.setAttribute ( "sqlOrder", "SELECT * from v_operation_display WHERE "+wc );

            SKGMainPanel::getMainPanel()->openPage ( SKGMainPanel::getMainPanel()->getPluginByName ( "Debug plugin" ), -1, doc.toString() );
        }
        else
        {
            //Call operation plugin
            QDomDocument doc ( "SKGML" );
            doc.setContent ( getDocument()->getParameter ( "SKGOPERATION_DEFAULT_PARAMETERS" ) );
            QDomElement root = doc.documentElement();
            if ( root.isNull() )
            {
                root=doc.createElement ( "parameters" );
                doc.appendChild ( root );
            }

            root.setAttribute ( "operationTable", "v_operation_display" );
            root.setAttribute ( "operationWhereClause", wc );
            root.setAttribute ( "title", title );
            root.setAttribute ( "title_icon", "chronometer" );
            root.setAttribute ( "currentPage", "-1" );

            SKGMainPanel::getMainPanel()->openPage ( SKGMainPanel::getMainPanel()->getPluginByName ( "Skrooge operation plugin" ), -1, doc.toString() );
        }
    }
}

void SKGScheduledPluginWidget::onSelectionChanged()
{
    SKGTRACEIN ( 10, "SKGScheduledPluginWidget::onSelectionChanged" );

    int nb=getNbSelectedObjects();
    ui.kFirstOccurenceDate->setEnabled ( nb<=1 );
    ui.kModifyBtn->setEnabled ( nb );
    ui.kProcessBtn->setEnabled ( nb );

    if ( nb==1 )
    {
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();
        SKGRecurrentOperationObject recOp=selection.at ( 0 );

        ui.kFirstOccurenceDate->setDate ( recOp.getDate() );
        ui.kOnceEveryVal->setValue ( recOp.getPeriodIncrement() );
        ui.kOnceEveryUnit->setCurrentIndex ( ( int ) recOp.getPeriodUnit() );

        ui.kRemindMeVal->setValue ( recOp.getWarnDays() );
        ui.kRemindMe->setCheckState ( recOp.isWarnEnabled() ? Qt::Checked : Qt::Unchecked );

        ui.kAutoWriteVal->setValue ( recOp.getAutoWriteDays() );
        ui.kAutoWrite->setCheckState ( recOp.isAutoWriteEnabled() ? Qt::Checked : Qt::Unchecked );

        ui.kNbTimesVal->setValue ( recOp.getTimeLimit() );
        ui.kNbTimes->setCheckState ( recOp.hasTimeLimit() ? Qt::Checked : Qt::Unchecked );
    }

    emit selectionChanged();
}

void SKGScheduledPluginWidget::onNbOccurrenceChanged()
{
    QDate firstDate=ui.kFirstOccurenceDate->date();
    SKGRecurrentOperationObject::PeriodUnit punit= ( SKGRecurrentOperationObject::PeriodUnit ) ui.kOnceEveryUnit->currentIndex();
    int p=ui.kOnceEveryVal->value();

    if ( ui.kLastOccurenceDate==this->sender() )
    {
        //End date has been modified.
        //We must set the number of occurrence
        QDate lastDate=ui.kLastOccurenceDate->date();
        if ( lastDate<=firstDate )
        {
            ui.kLastOccurenceDate->setDate ( firstDate );
            ui.kNbTimesVal->setValue ( 1 );
        }
        else
        {
            int nbd=firstDate.daysTo ( lastDate );
            if ( punit==SKGRecurrentOperationObject::DAY ) nbd=nbd/p;
            else if ( punit==SKGRecurrentOperationObject::MONTH )
            {
                nbd= ( lastDate.day() >=firstDate.day() ? 0 : -1 ) + ( lastDate.year()-firstDate.year() ) *12+ ( lastDate.month()-firstDate.month() );
            }
            else if ( punit==SKGRecurrentOperationObject::YEAR ) nbd=nbd/ ( 365*p );

            bool previous=ui.kNbTimesVal->blockSignals ( true );
            ui.kNbTimesVal->setValue ( nbd+1 );
            ui.kNbTimesVal->blockSignals ( previous );
        }

    }
    else
    {
        //We must compute the date
        p*= ( ui.kNbTimesVal->value()-1 );
        if ( punit==SKGRecurrentOperationObject::DAY ) firstDate=firstDate.addDays ( p );
        else if ( punit==SKGRecurrentOperationObject::MONTH ) firstDate=firstDate.addMonths ( p );
        else if ( punit==SKGRecurrentOperationObject::YEAR ) firstDate=firstDate.addYears ( p );

        bool previous=ui.kLastOccurenceDate->blockSignals ( true );
        ui.kLastOccurenceDate->setDate ( firstDate );
        ui.kLastOccurenceDate->blockSignals ( previous );
    }
}

void SKGScheduledPluginWidget::onUpdate()
{
    SKGError err;
    SKGTRACEINRC ( 10, "SKGScheduledPluginWidget::onUpdate",err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        //Get Selection
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

        int nb=selection.count();
        SKGBEGINPROGRESSTRANSACTION(*getDocument(), i18nc("Noun, name of the user action", "Recurrent operation update" ), err, nb );
        for ( int i=0; err.isSucceeded() && i<nb; ++i )
        {
            //Get the real object, not the object from the view
            SKGRecurrentOperationObject recOp=SKGRecurrentOperationObject ( selection.at ( i ).getDocument(), selection.at ( i ).getID() );

            //Update it
            if ( nb==1 ) err=recOp.setDate ( ui.kFirstOccurenceDate->date() );
            if ( err.isSucceeded() ) err=recOp.setPeriodIncrement ( ui.kOnceEveryVal->value() );
            if ( err.isSucceeded() ) err=recOp.setPeriodUnit ( ( SKGRecurrentOperationObject::PeriodUnit ) ui.kOnceEveryUnit->currentIndex() );
            if ( err.isSucceeded() ) err=recOp.setWarnDays ( ui.kRemindMeVal->value() );
            if ( err.isSucceeded() ) err=recOp.warnEnabled ( ui.kRemindMe->checkState() ==Qt::Checked );
            if ( err.isSucceeded() ) err=recOp.setAutoWriteDays ( ui.kAutoWriteVal->value() );
            if ( err.isSucceeded() ) err=recOp.autoWriteEnabled ( ui.kAutoWrite->checkState() ==Qt::Checked );
            if ( err.isSucceeded() ) err=recOp.setTimeLimit ( ui.kNbTimesVal->value() );
            if ( err.isSucceeded() ) err=recOp.timeLimit ( ui.kNbTimes->checkState() ==Qt::Checked );
            if ( err.isSucceeded() ) err=recOp.save();

            if ( err.isSucceeded() ) err=getDocument()->stepForward ( i+1 );
        }
        //status bar
        if ( err.isSucceeded() ) err=SKGError(0, i18nc("Successful message after an user action", "Recurrent operation updated." ) );
        else err.addError ( ERR_FAIL, i18nc("Error message",  "Update failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGScheduledPluginWidget::onProcess()
{
    SKGError err;
    SKGTRACEINRC ( 10, "SKGScheduledPluginWidget::onProcess",err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        //Get Selection
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

        int nb=selection.count();
        SKGBEGINPROGRESSTRANSACTION(*getDocument(), i18nc("Noun, name of the user action", "Insert recurrent operations" ), err, nb );
        for ( int i=0; err.isSucceeded() && i<nb; ++i )
        {
            //Get the real object, not the object from the view
            SKGRecurrentOperationObject recOp=SKGRecurrentOperationObject ( selection.at ( i ).getDocument(), selection.at ( i ).getID() );

            //Process it
            int nbi=0;
            err=recOp.process ( nbi, true );
            if ( err.isSucceeded() ) err=getDocument()->stepForward ( i+1 );
        }
        //status bar
        if ( err.isSucceeded() ) err=SKGError(0, i18nc("Successful message after an user action", "Recurrent operation inserted." ) );
        else err.addError ( ERR_FAIL, i18nc("Error message",  "Insertion failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

QWidget* SKGScheduledPluginWidget::zoomableWidget() const
{
    return ui.kView;
}
#include "skgscheduledpluginwidget.moc"
